from MuonDataLib.data.events.instrument import Instrument
import h5py
import numpy as np


class LoadEventData(object):
    """
    A class for loading event data.
    This includes methods for updating the data
    from a file that is being written.
    Can also get histograms for every detector.

    Methods:
    - set_bin_width
    - get_histograms
    - reload_data
    - load_data
    """

    def __init__(self):
        self._inst = None
        self._file_name = None

    def set_bin_width(self, width):
        """
        Sets the bin width for the histograms.
        :param width: the bin width to use (constant value).
        """
        self._inst.set_bin_width(width)

    def get_histograms(self):
        """
        Gets the histograms for every detector in the instrument.
        :returns: a list of the histograms and the bin edges
        """
        # loop periods - assume 1 for now
        histograms = []

        # loop over detectors
        for k in range(self._inst.N_det):
            y, x = self._inst.get_histogram(k)
            histograms.append(y)
        return [histograms], x

    def reload_data(self):
        """
        Updates the data from a currently active file
        i.e. updates current run.
        It uses the file name from load_data.
        """
        with h5py.File(self._file_name, 'r') as file:
            tmp = file.require_group('raw_data_1')
            tmp = tmp.require_group('detector_1')
            IDs = tmp['event_id'][:]
            start_indicies = tmp['event_index'][:]
            times = tmp['event_time_offset'][:]
            start_times = tmp['event_time_zero'][:]
            periods = tmp['period_number'][:]
            amps = tmp['pulse_height'][:]
            # add data
            self._inst.add_event_data(IDs,
                                      times,
                                      amps,
                                      periods,
                                      start_times,
                                      start_indicies)

    def load_data(self, file_name):
        """
        Loads event data.
        :param file_name: the name of the file to load.
        """
        self._file_name = file_name
        with h5py.File(self._file_name, 'r') as file:
            tmp = file.require_group('raw_data_1')
            tmp = tmp.require_group('detector_1')
            IDs = tmp['event_id'][:]
            start_indicies = tmp['event_index'][:]
            times = tmp['event_time_offset'][:]
            start_times = tmp['event_time_zero'][:]
            periods = tmp['period_number'][:]
            amps = tmp['pulse_height'][:]
            start = tmp['event_time_zero'].attrs['offset'][0]

            self._inst = Instrument(start, np.max(IDs) + 1)
            # add frame
            self._inst.add_new_frame(start_times[0],
                                     periods[0],
                                     start_indicies[0])
            # add data
            self._inst.add_event_data(IDs,
                                      times,
                                      amps,
                                      periods,
                                      start_times,
                                      start_indicies)



from MuonDataLib.data.sample import read_sample_from_histogram
from MuonDataLib.data.raw_data import read_raw_data_from_histogram
from MuonDataLib.data.source import read_source_from_histogram
from MuonDataLib.data.user import read_user_from_histogram
from MuonDataLib.data.periods import read_periods_from_histogram
from MuonDataLib.data.detector1 import read_detector1_from_histogram
from MuonDataLib.data.muon_data import MuonData
import h5py


def load_nxs2(file_name):
    with h5py.File(file_name, 'r') as file:
        return MuonData(sample=read_sample_from_histogram(file),
                        raw_data=read_raw_data_from_histogram(file),
                        source=read_source_from_histogram(file),
                        user=read_user_from_histogram(file),
                        periods=read_periods_from_histogram(file),
                        detector1=read_detector1_from_histogram(file))



import plotly.graph_objects as go


class Figure(object):
    """
    A simple class to make plotting easier.
    Uses plotly under the hood.
    """
    def __init__(self, title='',
                 x_label='time (micro seconds)',
                 y_label='Counts (per micro second)'):
        """
        Creates a figure for plotting.
        :param title: the title for the plot.
        :param x_label: the x label for the plot.
        :param y_label: the y label for the plot.
        """
        self._fig = go.Figure()
        self._fig.update_layout(title=title,
                                xaxis_title=x_label,
                                yaxis_title=y_label)

    def plot(self, bin_edges, y_values, label):
        """
        Adds data to the plot as point data.
        :param bin_edges: the bin edges
        :param y_values: the y values for the histogram
        :param label:  the label for the line
        """
        bin_centres = (bin_edges[:-1] + bin_edges[1:])/2.
        self._fig.add_trace(go.Scatter(x=bin_centres,
                                       y=y_values,
                                       mode='lines+markers',
                                       name=label))

    def show(self):
        """
        Shows the figure (opens in browser)
        """
        self._fig.show()

    def plot_from_instrument(self,
                             inst,
                             det_list,
                             label,
                             period=0):
        """
        Plots the data from an instrument
        :param inst: the instrument object to plot.
        :param det_list: list of the detectors to plot
        :param period: the period to plot
        """
        hists, edges = inst.get_histograms()
        for det in det_list:
            self.plot(edges, hists[period][det],
                      label + f'Detector {det}')



from MuonDataLib.data.detector1 import (Detector_1,
                                        read_detector1_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import os
import unittest
import numpy as np


FILENAME = 'detector1_test.nxs'


def create_single_period_data():
    x = [1., 2., 3.]
    # counts: shape(periods, spec, x)
    counts = []
    for p in range(1):
        counts.append([])
        for j in range(4):
            tmp = (((j + 1) * np.ones(len(x))))
            counts[p].append([int(x) for x in tmp])

    return Detector_1(1, x, [4, 5, 6, 7], counts, 'python', 3, 4, 42)


def create_multiperiod_data():
    x = [1., 2., 3.]
    # counts: shape(periods, spec, x)
    counts = []
    for p in range(3):
        counts.append([])
        for j in range(2):
            tmp = (((p + 1) * np.ones(len(x))))
            counts[p].append([int(x) for x in tmp])

    return Detector_1(1, x, [4, 5], counts, 'python', 3, 4, 42)


class Detector1Test(TestHelper):

    def test_detector1_object_stores_correct_info_single_period(self):
        """
        Check the class stores data correctly
        """
        det = create_single_period_data()

        self.assertEqual(det._dict['resolution'], 1)
        self.assertArrays(det._dict['raw_time'], [1, 2, 3])
        self.assertArrays(det._dict['spectrum_index'], [4, 5, 6, 7])
        self.assertEqual(det._dict['inst'], 'python')
        self.assertEqual(det._dict['time_zero'], 3)
        self.assertEqual(det._dict['first_good'], 4)
        self.assertEqual(det._dict['last_good'], 42)
        self.assertArrays(det._dict['counts'], [
                                                [[1, 1, 1],
                                                 [2, 2, 2],
                                                 [3, 3, 3],
                                                 [4, 4, 4]]])
        self.assertEqual(det.N_x, 3)
        self.assertEqual(det.N_hist, 4)
        self.assertEqual(det.N_periods, 1)

    def test_detector1_object_saves_correct_info_single_period(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        det = create_single_period_data()

        with h5py.File(FILENAME, 'w') as file:
            det.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            inst = file[keys[0]]

            keys = self.compare_keys(inst, ['instrument'])
            inst = inst[keys[0]]
            self.assertEqual(inst.attrs['NX_class'], 'NXinstrument')

            keys = self.compare_keys(inst, ['detector_1'])
            group = inst[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXdetector')

            keys = self.compare_keys(group, ['resolution',
                                             'raw_time',
                                             'spectrum_index',
                                             'counts',
                                             ])

            # check values
            self.assertEqual(group['resolution'][0], 1e6)
            self.assertArrays(group['raw_time'][:], [1., 2., 3.])
            self.assertArrays(group['spectrum_index'][:], [4, 5, 6, 7])
            self.assertArrays(group['counts'][:], [
                                                   [[1, 1, 1],
                                                    [2, 2, 2],
                                                    [3, 3, 3],
                                                    [4, 4, 4]]])

            # check attributes
            tmp = group['resolution']
            print(tmp.attrs['units'])
            self.assertEqual(tmp.attrs['units'].decode(), "picoseconds")

            tmp = group['raw_time']
            self.assertEqual(tmp.attrs['units'].decode(), 'microseconds')
            self.assertEqual(tmp.attrs['long_name'].decode(), 'time')

            tmp = group['counts']
            self.assertEqual(tmp.attrs['axes'].decode(),
                             '[period index, spectrum index, raw time bin]')
            self.assertEqual(tmp.attrs['long_name'].decode(), 'python')
            self.assertEqual(tmp.attrs['t0_bin'], 3)
            self.assertEqual(tmp.attrs['first_good_bin'], 4)
            self.assertEqual(tmp.attrs['last_good_bin'], 42)
        os.remove(FILENAME)

    def test_load_detector1_gets_correct_info_single_period(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        det = create_single_period_data()

        with h5py.File(FILENAME, 'w') as file:
            det.save_nxs2(file)
        del det

        with h5py.File(FILENAME, 'r') as file:
            load_det = read_detector1_from_histogram(file)

        self.assertEqual(load_det._dict['resolution'], 1)
        self.assertArrays(load_det._dict['raw_time'], [1, 2, 3])
        self.assertArrays(load_det._dict['spectrum_index'], [4, 5, 6, 7])
        self.assertEqual(load_det._dict['inst'], 'python')
        self.assertEqual(load_det._dict['time_zero'], 3)
        self.assertEqual(load_det._dict['first_good'], 4)
        self.assertEqual(load_det._dict['last_good'], 42)
        self.assertArrays(load_det._dict['counts'], [
                                                     [[1, 1, 1],
                                                      [2, 2, 2],
                                                      [3, 3, 3],
                                                      [4, 4, 4]]])
        self.assertEqual(load_det.N_x, 3)
        self.assertEqual(load_det.N_hist, 4)
        self.assertEqual(load_det.N_periods, 1)

        os.remove(FILENAME)

    def test_detector1_object_stores_correct_info_multiperiod(self):
        """
        Check the class stores data correctly
        """
        det = create_multiperiod_data()

        self.assertEqual(det._dict['resolution'], 1)
        self.assertArrays(det._dict['raw_time'], [1., 2., 3.])
        self.assertArrays(det._dict['spectrum_index'], [4, 5])
        self.assertEqual(det._dict['inst'], 'python')
        self.assertEqual(det._dict['time_zero'], 3)
        self.assertEqual(det._dict['first_good'], 4)
        self.assertEqual(det._dict['last_good'], 42)
        print(det._dict['counts'])
        self.assertArrays(det._dict['counts'], [
                                                [[1, 1, 1], [1, 1, 1]],
                                                [[2, 2, 2], [2, 2, 2]],
                                                [[3, 3, 3], [3, 3, 3]]])

        self.assertEqual(det.N_x, 3)
        self.assertEqual(det.N_hist, 2)
        self.assertEqual(det.N_periods, 3)

    def test_detector1_object_saves_correct_info_multiperiod(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        det = create_multiperiod_data()

        with h5py.File(FILENAME, 'w') as file:
            det.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            inst = file[keys[0]]

            keys = self.compare_keys(inst, ['instrument'])
            inst = inst[keys[0]]
            self.assertEqual(inst.attrs['NX_class'], 'NXinstrument')

            keys = self.compare_keys(inst, ['detector_1'])
            group = inst[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXdetector')

            keys = self.compare_keys(group, ['resolution',
                                             'raw_time',
                                             'spectrum_index',
                                             'counts',
                                             ])

            # check values
            self.assertEqual(group['resolution'][0], 1e6)
            self.assertArrays(group['raw_time'][:], [1., 2., 3.])
            self.assertArrays(group['spectrum_index'][:], [4, 5])
            self.assertArrays(group['counts'][:], [
                                                   [[1, 1, 1], [1, 1, 1]],
                                                   [[2, 2, 2], [2, 2, 2]],
                                                   [[3, 3, 3], [3, 3, 3]]])

            # check attributes
            tmp = group['resolution']
            print(tmp.attrs['units'])
            self.assertEqual(tmp.attrs['units'].decode(), "picoseconds")

            tmp = group['raw_time']
            self.assertEqual(tmp.attrs['units'].decode(), 'microseconds')
            self.assertEqual(tmp.attrs['long_name'].decode(), 'time')

            tmp = group['counts']
            self.assertEqual(tmp.attrs['axes'].decode(),
                             '[period index, spectrum index, raw time bin]')
            self.assertEqual(tmp.attrs['long_name'].decode(), 'python')
            self.assertEqual(tmp.attrs['t0_bin'], 3)
            self.assertEqual(tmp.attrs['first_good_bin'], 4)
            self.assertEqual(tmp.attrs['last_good_bin'], 42)
        os.remove(FILENAME)

    def test_load_detector1_gets_correct_info_multiperiod(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        det = create_multiperiod_data()

        with h5py.File(FILENAME, 'w') as file:
            det.save_nxs2(file)
        del det

        with h5py.File(FILENAME, 'r') as file:
            load_det = read_detector1_from_histogram(file)

        self.assertEqual(load_det._dict['resolution'], 1)
        self.assertArrays(load_det._dict['raw_time'], [1, 2, 3])
        self.assertArrays(load_det._dict['spectrum_index'], [4, 5])
        self.assertEqual(load_det._dict['inst'], 'python')
        self.assertEqual(load_det._dict['time_zero'], 3)
        self.assertEqual(load_det._dict['first_good'], 4)
        self.assertEqual(load_det._dict['last_good'], 42)
        self.assertArrays(load_det._dict['counts'], [
                                                     [[1, 1, 1], [1, 1, 1]],
                                                     [[2, 2, 2], [2, 2, 2]],
                                                     [[3, 3, 3], [3, 3, 3]]])
        self.assertEqual(load_det.N_x, 3)
        self.assertEqual(load_det.N_hist, 2)
        self.assertEqual(load_det.N_periods, 3)

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.events.detector import Detector
from MuonDataLib.test_helpers.unit_test import TestHelper
import unittest


class DetectorTest(TestHelper):

    def test_init(self):
        det = Detector(5)

        self.assertEqual(det._ID, 5)
        self.assertEqual(det._frames, {})
        self.assertEqual(det._current, -1)

    def test_add_new_frame(self):
        det = Detector(4)

        self.assertEqual(det._frames, {})

        det.add_new_frame(1.2, 0)
        self.assertEqual(len(det._frames), 1)
        self.assertEqual(list(det._frames.keys())[0], 0)
        self.assertEqual(det._current, 0)

        frame = det._frames[0]
        self.assertAlmostEqual(frame._start, 1.2)
        self.assertEqual(frame._period, 0)
        self.assertEqual(frame._event_times, [])
        self.assertEqual(frame._event_amplitudes, [])

    def test_add_events_to_frame(self):
        det = Detector(4)
        det.add_new_frame(1.2, 0)

        det.add_events_to_frame(0, [1, 2, 3], [3, 2, 1])

        frame = det._frames[0]
        self.assertAlmostEqual(frame._start, 1.2)
        self.assertEqual(frame._period, 0)
        self.assertArrays(frame._event_times, [1, 2, 3])
        self.assertArrays(frame._event_amplitudes, [3, 2, 1])

    def test_two_add_events_to_frame(self):
        det = Detector(4)
        det.add_new_frame(1.2, 0)
        det.add_events_to_frame(0, [1, 2, 3], [3, 2, 1])
        frame = det._frames[0]
        self.assertArrays(frame._event_times, [1, 2, 3])
        self.assertArrays(frame._event_amplitudes, [3, 2, 1])

        det.add_events_to_frame(0, [1, 2, 3, 4, 5, 6], [3, 2, 1, 6, 5, 4])
        frame = det._frames[0]
        self.assertArrays(frame._event_times, [1, 2, 3, 4, 5, 6])
        self.assertArrays(frame._event_amplitudes, [3, 2, 1, 6, 5, 4])

    def test_get_histogram(self):
        det = Detector(0)
        det.add_new_frame(0, 0)
        det.add_events_to_frame(0,
                                [1, 1.2, 2.3, 2.4, 5.],
                                [3, 2, 1, 1, 2])
        bin_edges = [0, 1, 2, 3, 4, 5, 6]
        hist, bins = det.get_histogram(bin_edges)
        self.assertArrays(bin_edges, bins)
        self.assertArrays(hist, [0, 2, 2, 0, 0, 1])

    def test_get_histogram_conversion(self):
        det = Detector(0)
        det.add_new_frame(0, 0)
        det.add_events_to_frame(0,
                                [1, 1.2, 2.3, 2.4, 5.],
                                [3, 2, 1, 1, 2])
        bin_edges = [0, 1, 2, 3, 4, 5, 6]
        hist, bins = det.get_histogram(bin_edges, 0.5)
        self.assertArrays(bin_edges, bins)
        self.assertArrays(hist, [2, 2, 1, 0, 0, 0])


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.user import (User,
                                   read_user_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import unittest
import os


FILENAME = 'user_test.nxs'


class UserTest(TestHelper):

    def test_user_object_stores_correct_info(self):
        """
        Check the class stores data correctly
        """
        user = User('Unit Test', 'Python')

        self.assertEqual(user._dict['name'], 'Unit Test')
        self.assertEqual(user._dict['affiliation'], 'Python')

    def test_user_object_saves_correct_info(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        user = User('Unit Test', 'Python')

        with h5py.File(FILENAME, 'w') as file:
            user.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])

            group = file[keys[0]]

            keys = self.compare_keys(group, ['user_1'])
            group = group[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXuser')

            keys = self.compare_keys(group, ['name', 'affiliation'])

            self.assertString(group, 'name', 'Unit Test')
            self.assertString(group, 'affiliation', 'Python')

        os.remove(FILENAME)

    def test_load_user_gets_correct_info(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        user = User('Unit Test', 'Python')

        with h5py.File(FILENAME, 'w') as file:
            user.save_nxs2(file)
        del user

        with h5py.File(FILENAME, 'r') as file:
            load_user = read_user_from_histogram(file)

        self.assertEqual(load_user._dict['name'], 'Unit Test')
        self.assertEqual(load_user._dict['affiliation'], 'Python')

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import unittest


class TestHelper(unittest.TestCase):
    """
    A simple wrapper to make unit tests easier
    for nxs file
    """
    def compare_keys(self, nxs, expected):
        """
        Compares the options (keys) from a nexus file
        to an expected list
        :param nxs: the open nexus file (at the correct level)
        :param expected: the list of expected keys
        :return: the keys from the nexus file
        """
        keys = list(nxs.keys())
        # check same number of keys
        self.assertEqual(len(expected), len(keys))
        ref = expected
        # check keys match
        for value in keys:
            self.assertTrue(value in ref)
            ref.remove(value)
        # check all of the expected values have been seen
        self.assertEqual(len(ref), 0)
        return keys

    def assertString(self, group, key, expected):
        """
        The strings in nexus files are lists,
        with just one element. They also need
        decoding
        :param group: the open nexus group
        :param key: the key we want to check
        :param expected: the expected string
        """
        string_list = group[key]
        self.assertEqual(len(string_list), 1)
        self.assertEqual(string_list[0].decode(), expected)

    def assertArrays(self, array, ref):
        for j in range(len(array)):
            len_a = len(array)
            len_r = len(ref)
            msg = f'The arrays are not the same length: {len_a}, {len_r}'
            self.assertEqual(len_a, len_r, msg=msg)

            if isinstance(array[j], (list, np.ndarray)):
                self.assertArrays(array[j], ref[j])
            else:
                msg = f'values do not match in array {array[j]}, {ref[j]}'
                self.assertAlmostEqual(array[j], ref[j], 3, msg=msg)



import argparse
import sys


VERSION_STRING = "version = "
VERSION_MARKS = '"'
ORDER = ['major', 'minor', 'patch', 'beta']
VERSION_SEP = ['.', '.', 'b', '']


def make_str(version):
    """
    Simple function to convert version dict into a string
    """
    txt = VERSION_MARKS
    for j in range(len(ORDER)):
        txt += version[ORDER[j]] + VERSION_SEP[j]
    txt += VERSION_MARKS + '\n'
    return txt


def update_version_str(txt: str, bump: str) -> str:
    """
    Extracts the version from text and then applies
    an appropriate bump
    """
    tmp = txt.split(VERSION_MARKS)
    tmp = tmp[1]
    version = {}

    version[ORDER[0]] = tmp.split(VERSION_SEP[0])[0]
    version[ORDER[1]] = tmp.split(VERSION_SEP[1])[1]
    version[ORDER[2]] = tmp.split(VERSION_SEP[2])[0].split(VERSION_SEP[1])[2]
    version[ORDER[3]] = tmp.split(VERSION_SEP[2])[1]

    version[bump] = str(int(version[bump]) + 1)
    index = ORDER.index(bump)
    # reset below current bump
    for j in range(index+1, len(ORDER)):
        version[ORDER[j]] = str(0)
    return VERSION_STRING + make_str(version)


def update_version(file_name: str, bump: str):
    """
    Reads the file and then updates the version
    according to the specified bump
    """
    if bump not in ORDER:
        raise ValueError("Specified bump is not allowed")

    lines = []
    with open(file_name, 'r') as file:
        lines = file.readlines()

    with open(file_name, 'w') as file:
        for txt in lines:
            if VERSION_STRING in txt:
                txt = update_version_str(txt, bump)
            file.write(txt)


def get_input():
    """
    get the bump for the version from the command line arg
    :returns the bump for the version
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('bump',
                        help='the bump to the setup file', type=str)
    args = parser.parse_args()
    print(args)
    return args.bump


if __name__ == "__main__":
    try:
        bump = get_input()
        update_version('pyproject.toml', bump)
    except ValueError:
        error = sys.exc_info()[1]
        print(error)



"""
Written a custom parser to write yml file
the default did not have nice formatting
and didn't allow for nice managment of
dependencies

f strings have not been used as the mac github action
does not recognise them and complains about
syntax errors.
"""


INDENT = ' '


def record_name(yml, outfile):
    """
    Adds the name tag from the yml to the file
    :param yml: a dict containing the desired contents of the yml file
    :param outfile: the file the data will be written to
    """
    outfile.write('name: ' + str(yml['name']) + ' \n \n')


def record_channels(yml, outfile):
    """
    Adds the channels tag from the yml to the file
    :param yml: a dict containing the desired contents of the yml file
    :param outfile: the file the data will be written to
    """
    outfile.write('channels: \n')
    outfile.write(str(INDENT) + ' - ' + str(yml['channels']) + ' \n \n')


def record_pip(pip_dict, outfile):
    """
    Adds the packages to be installed by pip to the file
    :param pip_dict: a dict containing the packages for pip
    :param outfile: the file the data will be written to
    """
    for package in pip_dict.keys():
        big_indent = INDENT + INDENT + INDENT
        outfile.write(str(big_indent) + ' - ' + str(package) +
                      ' ' + str(pip_dict[package]) + ' \n')


def record_dependencies(yml, outfile):
    """
    Adds the dependencies to be listed in the file
    :param yml: a dict containing the desired contents of the yml file
    :param outfile: the file the data will be written to
    """

    outfile.write('dependencies: \n')

    deps = yml['dependencies']
    for package in deps.keys():
        if package == 'pip' and deps['pip'] != {}:
            outfile.write(str(INDENT) + ' - pip: \n')
            record_pip(deps['pip'], outfile)
        elif package != 'pip':
            outfile.write(str(INDENT) + ' - ' + str(package) +
                          ' ' + str(deps[package]) + ' \n')


def write_conda_yml_from_dict(yml, outfile):
    """
    Writes the file based on the input dict yml
    :param yml: a dict containing the desired contents of the yml file
    :param outfile: the file the data will be written to
    """

    record_name(yml, outfile)
    record_channels(yml, outfile)

    record_dependencies(yml, outfile)



from MuonDataLib.data.loader.load_event import LoadEventData
from MuonDataLib.test_helpers.unit_test import TestHelper
from MuonDataLib.test_helpers.utils import extract_event_data
import numpy as np
import unittest
import os


class LoadEventDataTest(TestHelper):

    def assertEvents(self, inst, ID, frame, times, amps, period, start):
        frame = extract_event_data(inst, ID, frame)
        self.assertArrays(frame.get_event_times, times)
        self.assertArrays(frame.get_event_amplitudes, amps)
        self.assertEqual(frame.get_period, period)
        self.assertEqual(frame.get_start_time, start)

    def test_load_event_data(self):
        """
        """
        file = os.path.join(os.path.dirname(__file__),
                            '..',
                            '..',
                            'data_files',
                            'simple_test.nxs')
        load = LoadEventData()
        load.load_data(file)

        inst = load._inst

        self.assertEvents(inst, 0, 0,
                          [16, 602, 3846],
                          [10, 10, 10],
                          0,
                          0)

        self.assertEvents(inst, 60, 0,
                          [95, 836, 1055, 2734],
                          [10, 10, 10, 10],
                          0,
                          0)

        self.assertEvents(inst, 0, 1,
                          [127, 425, 491, 907, 957, 1207],
                          [10, 10, 10, 10, 10, 10],
                          0,
                          20000000)

    def test_reload_data(self):
        file_part = os.path.join(os.path.dirname(__file__),
                                 '..',
                                 '..',
                                 'data_files',
                                 'partial_run.nxs')
        file_full = os.path.join(os.path.dirname(__file__),
                                 '..',
                                 '..',
                                 'data_files',
                                 'complete_run.nxs')

        full_data = LoadEventData()
        full_data.load_data(file_full)

        load = LoadEventData()
        load.load_data(file_part)
        """
        since the file doesn't actually update
        we will manually change the file
        """
        load._file_name = file_full
        # check partial data is shorter
        self.assertLess(load._inst._current_frame,
                        full_data._inst._current_frame)

        load.reload_data()
        # check both end in the same state
        self.assertEqual(full_data._inst._start,
                         load._inst._start)

        self.assertEqual(full_data._inst._current_frame,
                         load._inst._current_frame)

        self.assertEqual(full_data._inst._current_index,
                         load._inst._current_index)

        for det in range(len(load._inst._detectors)):
            det_full = full_data._inst._detectors[det]
            det_part = load._inst._detectors[det]
            for frame in det_full._frames.keys():
                full_frame = det_full._frames[frame]
                part_frame = det_part._frames[frame]

                self.assertEqual(full_frame._start,
                                 part_frame._start)
                self.assertEqual(full_frame._period,
                                 part_frame._period)
                self.assertArrays(full_frame.get_event_times,
                                  part_frame.get_event_times)
                self.assertArrays(full_frame.get_event_amplitudes,
                                  part_frame.get_event_amplitudes)

    def test_get_histograms(self):
        file_full = os.path.join(os.path.dirname(__file__),
                                 '..',
                                 '..',
                                 'data_files',
                                 'simple_test.nxs')

        full_data = LoadEventData()
        full_data.load_data(file_full)
        hist, bins = full_data.get_histograms()
        self.assertArrays(bins, np.arange(0, 30.5, .5))
        self.assertEqual(len(hist[0]), 64)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.utils import (convert_date_for_NXS,
                                    convert_date)

from MuonDataLib.data.hdf5 import HDF5


class RawData(HDF5):
    """
    A class for storing the "raw data" information for a
    muon nexus v2 file
    """
    def __init__(self, good_frames, IDF, definition, inst, title, notes,
                 run_number, duration, raw_frames, start_time, end_time, ID):
        """
        Creates the raw data object, which holds all of the information
        for the raw data section of a muon nexus v2 file
        :param good_frames: the number of good frames
        :param IDF: the version of the Instrument Definition File (IDF)
        :param definition: the definition (e.g. pulsedTD)
        :param inst: the instrument name
        :param title: the title of the experiment
        :param notes: the additional notes/comments
        :param run_number: the run number
        :param duration: the duration of the experiment
        :param raw_frames: the number of raw frames
        :param start_time: the start time of the experiment
        :param end_time: the end time of the experiment
        :param ID: a string identifier for the experiment
        """
        super().__init__()
        self._dict['good_frames'] = good_frames
        self._dict['IDF'] = IDF
        self._dict['def'] = definition
        self._dict['inst'] = inst
        self._dict['title'] = title
        self._dict['notes'] = notes
        self._dict['run_number'] = run_number
        self._dict['duration'] = duration
        self._dict['raw_frames'] = raw_frames
        self._dict['start'] = start_time
        self._dict['end'] = end_time
        self._dict['ID'] = ID

    def save_nxs2(self, file):
        """
        A method to save the information into a muon nexus v2 file
        :param file: the open file to write to.
        """
        tmp = file.require_group('raw_data_1')
        tmp.attrs['NX_class'] = 'NXentry'

        self.save_int('good_frames', self._dict['good_frames'], tmp)
        self.save_int('IDF_version', self._dict['IDF'], tmp)
        self.save_str('definition', self._dict['def'], tmp)
        self.save_str('name', self._dict['inst'], tmp)
        self.save_str('title', self._dict['title'], tmp)
        self.save_str('notes', self._dict['notes'], tmp)
        self.save_int('run_number', self._dict['run_number'], tmp)
        self.save_float('duration', self._dict['duration'], tmp)
        self.save_int('raw_frames', self._dict['raw_frames'], tmp)
        self.save_str('start_time',
                      convert_date_for_NXS(self._dict['start']),
                      tmp)
        self.save_str('end_time', convert_date_for_NXS(self._dict['end']), tmp)
        self.save_str('experiment_identifier', self._dict['ID'], tmp)

        tmp = tmp.require_group('instrument')
        self.save_str('name', self._dict['inst'], tmp)


def read_raw_data_from_histogram(file):
    """
    A function for reading the raw data information
    from a muon nexus v2 file
    :param file: the open file to read from
    :return: the RawData object
    """
    tmp = file['raw_data_1']
    return RawData(tmp['good_frames'][0],
                   tmp["IDF_version"][0],
                   tmp['definition'][0].decode(),
                   tmp['name'][0].decode(),
                   tmp['title'][0].decode(),
                   tmp['notes'][0].decode(),
                   tmp['run_number'][0],
                   tmp['duration'][0],
                   tmp['raw_frames'][0],
                   convert_date(tmp['start_time'][0].decode()),
                   convert_date(tmp['end_time'][0].decode()),
                   tmp['experiment_identifier'][0].decode())



from MuonDataLib.data.periods import (Periods,
                                      read_periods_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import unittest
import os


FILENAME = 'periods_test.nxs'


def create_single_period_data():
    return Periods(1, 'period 1', [1], [500], [1000], [1], [1.23], [2])


def create_multiperiod_data():
    return Periods(2, 'period 1; period 2', [1, 2], [500, 400],
                   [1000, 500], [1, 0], [1.23, 4.56], [42, 42])


class PeriodsTest(TestHelper):

    def test_periods_object_stores_correct_info_single_period(self):
        """
        Check the class stores data correctly
        """
        period = create_single_period_data()

        self.assertEqual(period._dict['number'], 1)
        self.assertEqual(period._dict['labels'], 'period 1')
        self.assertArrays(period._dict['type'], [1])
        self.assertArrays(period._dict['requested'], [500])
        self.assertArrays(period._dict['raw'], [1000])
        self.assertArrays(period._dict['output'], [1])
        self.assertArrays(period._dict['sequences'], [2])
        self.assertArrays(period._dict['counts'], [1.23])

    def test_periods_object_saves_correct_info_single_period(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        periods = create_single_period_data()

        with h5py.File(FILENAME, 'w') as file:
            periods.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            group = file[keys[0]]
            keys = self.compare_keys(group, ['periods'])
            group = group[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXperiod')

            self.assertEqual(group['number'][0], 1)
            self.assertString(group, 'labels', 'period 1')
            self.assertArrays(group['type'], [1])
            self.assertArrays(group['frames_requested'], [500])
            self.assertArrays(group['raw_frames'], [1000])
            self.assertArrays(group['output'], [1])
            self.assertArrays(group['sequences'], [2])
            self.assertArrays(group['total_counts'], [1.23])

        os.remove(FILENAME)

    def test_load_periods_gets_correct_info_single_period(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        period = create_single_period_data()

        with h5py.File(FILENAME, 'w') as file:
            period.save_nxs2(file)
        del period

        with h5py.File(FILENAME, 'r') as file:
            load_period = read_periods_from_histogram(file)

        self.assertEqual(load_period._dict['number'], 1)
        self.assertEqual(load_period._dict['labels'], 'period 1')
        self.assertArrays(load_period._dict['type'], [1])
        self.assertArrays(load_period._dict['requested'], [500])
        self.assertArrays(load_period._dict['raw'], [1000])
        self.assertArrays(load_period._dict['output'], [1])
        self.assertArrays(load_period._dict['sequences'], [2])
        self.assertArrays(load_period._dict['counts'], [1.23])

        os.remove(FILENAME)

    def test_periods_object_stores_correct_info_multiperiod(self):
        """
        Check the class stores data correctly
        """
        period = create_multiperiod_data()

        self.assertEqual(period._dict['number'], 2)
        self.assertEqual(period._dict['labels'], 'period 1; period 2')
        self.assertArrays(period._dict['type'], [1, 2])
        self.assertArrays(period._dict['requested'], [500, 400])
        self.assertArrays(period._dict['raw'], [1000, 500])
        self.assertArrays(period._dict['output'], [1, 0])
        self.assertArrays(period._dict['sequences'], [42, 42])
        self.assertArrays(period._dict['counts'], [1.23, 4.56])

    def test_periods_object_saves_correct_info_multiperiod(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        period = create_multiperiod_data()

        with h5py.File(FILENAME, 'w') as file:
            period.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            group = file[keys[0]]
            keys = self.compare_keys(group, ['periods'])
            group = group[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXperiod')

            self.assertEqual(group['number'][0], 2)
            self.assertEqual(group['labels'][0].decode(), 'period 1; period 2')
            self.assertArrays(group['type'], [1, 2])
            self.assertArrays(group['frames_requested'], [500, 400])
            self.assertArrays(group['raw_frames'], [1000, 500])
            self.assertArrays(group['output'], [1, 0])
            self.assertArrays(group['sequences'], [42, 42])
            self.assertArrays(group['total_counts'], [1.23, 4.56])

        os.remove(FILENAME)

    def test_load_detector1_gets_correct_info_multiperiod(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        period = create_multiperiod_data()

        with h5py.File(FILENAME, 'w') as file:
            period.save_nxs2(file)
        del period

        with h5py.File(FILENAME, 'r') as file:
            load_period = read_periods_from_histogram(file)

        self.assertEqual(load_period._dict['number'], 2)
        self.assertEqual(load_period._dict['labels'], 'period 1; period 2')
        self.assertArrays(load_period._dict['type'], [1, 2])
        self.assertArrays(load_period._dict['requested'], [500, 400])
        self.assertArrays(load_period._dict['raw'], [1000, 500])
        self.assertArrays(load_period._dict['output'], [1, 0])
        self.assertArrays(load_period._dict['sequences'], [42, 42])
        self.assertArrays(load_period._dict['counts'], [1.23, 4.56])

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.source import (Source,
                                     read_source_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import unittest
import os


FILENAME = 'source_test.nxs'


def create_data():
    return Source('python', 'muon', 'pulsed')


class SourceTest(TestHelper):

    def test_source_object_stores_correct_info(self):
        """
        Check the class stores data correctly
        """
        source = create_data()

        self.assertEqual(source._dict['name'], 'python')
        self.assertEqual(source._dict['probe'], 'muon')
        self.assertEqual(source._dict['type'], 'pulsed')

    def test_source_object_saves_correct_info(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        source = create_data()

        with h5py.File(FILENAME, 'w') as file:
            source.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['instrument'])
            group = group[keys[0]]

            keys = self.compare_keys(group, ['source'])
            group = group[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXsource')

            keys = self.compare_keys(group, ['name',
                                             'type',
                                             'probe'])

            self.assertString(group, 'name', 'python')
            self.assertString(group, 'probe', 'muon')
            self.assertString(group, 'type', 'pulsed')

        os.remove(FILENAME)

    def test_load_source_gets_correct_info(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        source = create_data()

        with h5py.File(FILENAME, 'w') as file:
            source.save_nxs2(file)

        del source

        with h5py.File(FILENAME, 'r') as file:
            load_source = read_source_from_histogram(file)

        self.assertEqual(load_source._dict['name'], 'python')
        self.assertEqual(load_source._dict['probe'], 'muon')
        self.assertEqual(load_source._dict['type'], 'pulsed')

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.hdf5 import HDF5


class Source(HDF5):
    """
    A class to store the source information needed
    for a muon nexus v2 file
    """
    def __init__(self, name, probe, s_type):
        """
        A small class to store the source information
        :param name: the source name (e.g. ISIS)
        :param probe: the type of probe used
        :param type: the type of source (e.g. pulsed)
        """
        super().__init__()
        self._dict['name'] = name
        self._dict['probe'] = probe
        self._dict['type'] = s_type

    def save_nxs2(self, file):
        """
        Method to save the source information to a
        muon nexus v2 file
        :param file: the open file to write to
        """
        tmp = file.require_group('raw_data_1')
        tmp = tmp.require_group('instrument')

        tmp = tmp.require_group('source')
        tmp.attrs['NX_class'] = 'NXsource'
        for key in self._dict.keys():
            self.save_str(key, self._dict[key], tmp)


def read_source_from_histogram(file):
    """
    A function to read the source information
    from a muon nexus v2 file
    :param file: the open file to read from
    :return: the source information
    """
    tmp = file['raw_data_1']['instrument']['source']
    return Source(tmp['name'][0].decode(),
                  tmp['probe'][0].decode(),
                  tmp['type'][0].decode())



from distutils.core import Extension
import numpy
from os.path import join
from typing import Sequence, Dict
from Cython.Build import cythonize
from pathlib import Path


def read_makefiles(package_name: str) -> Dict[str, str]:
    """
    Automatically locates all of the makefiles in src
    and reads the contents.
    :param package_name: the name of the package
    :return a dict of the python imports and file locations
    """
    makefile_data = []
    module_source_map = {}
    for path in Path('src').glob('**/*makefile.txt'):
        with open(path, 'r') as file:
            lines = file.readlines()
            for line in lines:
                makefile_data.append(line)

    for line in makefile_data:
        py_path, file_path = line.split(':')
        file_path = str(eval(file_path))
        module_source_map[f'{package_name}.{py_path}'] = [file_path]
    return module_source_map


def create_extension(fq_name: str,
                     sources: Sequence[str]) -> Extension:
    """
    Create an extension module
    :param fq_name: The final fully-qualified name of the module
    :param sources: List of relative paths from this file to the sources
    :return: An Extension class to be built
    """
    return Extension(name=fq_name,
                     sources=sources,
                     include_dirs=[numpy.get_include()])


def source_paths(dirname: str, filenames: Sequence[str]) -> Sequence[str]:
    """
    :param dirname: A relative path to the list of source files
    :param filenames: the names of the source files
    :return: A list of relative paths to the given sources in the directory
    """
    return [join(dirname, filename) for filename in filenames]


def get_extensions(package_name):
    module_source_map = read_makefiles(package_name)
    path = join('src', package_name)
    return cythonize([create_extension(name,
                      source_paths(str(path), sources)) for
                      name, sources in module_source_map.items()])



from MuonDataLib.data.raw_data import (RawData,
                                       read_raw_data_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import unittest
import os
import datetime


FILENAME = 'raw_data_test.nxs'


def create_data():
    start = datetime.datetime(2018, 12, 24, 13, 32, 1)
    end = datetime.datetime(2018, 12, 24, 18, 11, 52)

    return (RawData(10, 1, 'pulsed', 'python', 'raw data test',
                    'testing', 42, 1024.0, 51, start, end, '19'),
            start, end)


class RawDataTest(TestHelper):

    def test_raw_data_object_stores_correct_info(self):
        """
        Check the class stores data correctly
        """
        raw, start, end = create_data()

        self.assertEqual(raw._dict['good_frames'], 10)
        self.assertEqual(raw._dict['IDF'], 1)
        self.assertEqual(raw._dict['def'], 'pulsed')
        self.assertEqual(raw._dict['inst'], 'python')
        self.assertEqual(raw._dict['title'], 'raw data test')
        self.assertEqual(raw._dict['notes'], 'testing')
        self.assertEqual(raw._dict['run_number'], 42)
        self.assertAlmostEqual(raw._dict['duration'], 1024.0, 3)
        self.assertEqual(raw._dict['raw_frames'], 51)
        self.assertEqual(raw._dict['start'], start)
        self.assertEqual(raw._dict['end'], end)

    def test_raw_data_object_saves_correct_info(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        raw, _, _ = create_data()

        with h5py.File(FILENAME, 'w') as file:
            raw.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            group = file[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXentry')

            keys = self.compare_keys(group, ['good_frames',
                                             'IDF_version',
                                             'definition',
                                             'name',
                                             'title',
                                             'notes',
                                             'run_number',
                                             'duration',
                                             'raw_frames',
                                             'start_time',
                                             'end_time',
                                             'experiment_identifier',
                                             'instrument'])

            self.assertArrays(group['good_frames'], [10])
            self.assertArrays(group['IDF_version'], [1])
            self.assertString(group, 'definition', 'pulsed')
            self.assertString(group, 'name', 'python')
            self.assertString(group, 'title', 'raw data test')
            self.assertString(group, 'notes', 'testing')
            self.assertArrays(group['run_number'], [42])
            self.assertArrays(group['duration'], [1024.0])
            self.assertArrays(group['raw_frames'], [51])
            self.assertString(group, 'start_time', '2018-12-24T13:32:01')
            self.assertString(group, 'end_time', '2018-12-24T18:11:52')

            group = group['instrument']
            self.compare_keys(group, ['name'])
            self.assertString(group, 'name', 'python')

        os.remove(FILENAME)

    def test_load_raw_data_gets_correct_info(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        raw, start, end = create_data()

        with h5py.File(FILENAME, 'w') as file:
            raw.save_nxs2(file)

        del raw

        with h5py.File(FILENAME, 'r') as file:
            load_raw = read_raw_data_from_histogram(file)

        self.assertEqual(load_raw._dict['good_frames'], 10)
        self.assertEqual(load_raw._dict['IDF'], 1)
        self.assertEqual(load_raw._dict['def'], 'pulsed')
        self.assertEqual(load_raw._dict['inst'], 'python')
        self.assertEqual(load_raw._dict['title'], 'raw data test')
        self.assertEqual(load_raw._dict['notes'], 'testing')
        self.assertEqual(load_raw._dict['run_number'], 42)
        self.assertAlmostEqual(load_raw._dict['duration'], 1024.0, 3)
        self.assertEqual(load_raw._dict['raw_frames'], 51)
        self.assertEqual(load_raw._dict['start'], start)
        self.assertEqual(load_raw._dict['end'], end)

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import datetime


# some useful names
INT32 = np.int32
FLOAT32 = np.float32


def convert_date_for_NXS(date):
    """
    A method to change the date object into the
    format needed for the muon nexus v2 file
    :param data: the date object
    :return: a string of the date
    (<year>-<month>-<day>T<hour>:<min><sec>)
    """
    return date.strftime('%Y-%m-%dT%H:%M:%S')


def convert_date(date):

    """
    Convert the muon nexus v2 file data string into a
    date object.
    Assume in the form f'{year} {month} {day}', time
    :param date: the date string in the above format
    :return: the date object
    """

    return datetime.datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')


def stype(string):
    """
    A simple method for reporting
    the length of a string for saving
    to a muon nexus v2 file
    :param string: the string to be saved
    :return: a string that reports the length of the
    string
    """
    return 'S{}'.format(len(string)+1)



from MuonDataLib.data.events.frame import Frame
from MuonDataLib.test_helpers.unit_test import TestHelper
import unittest


class FrameTest(TestHelper):

    def test_init(self):
        frame = Frame(4.2, 1)

        self.assertEqual(frame._start, 4.2)
        self.assertEqual(frame._period, 1)
        self.assertEqual(frame._event_times, [])
        self.assertEqual(frame._event_amplitudes, [])

    def test_add_events(self):
        frame = Frame(3.1, 2)
        frame.add_events([1, 2, 3], [3, 2, 1])
        self.assertArrays([1, 2, 3], frame._event_times)
        self.assertArrays([3, 2, 1], frame._event_amplitudes)

    def test_add_two_sets_of_events(self):
        frame = Frame(3.1, 2)
        frame.add_events([1, 2, 3], [3, 2, 1])
        self.assertArrays([1, 2, 3], frame._event_times)
        self.assertArrays([3, 2, 1], frame._event_amplitudes)
        # add the full array (should remove the already stored data)
        frame.add_events([1, 2, 3, 4, 5, 6], [3, 2, 1, 6, 5, 4])
        self.assertArrays([1, 2, 3, 4, 5, 6], frame._event_times)
        self.assertArrays([3, 2, 1, 6, 5, 4], frame._event_amplitudes)

    def test_clear(self):
        frame = Frame(3.1, 2)
        frame.add_events([1, 2, 3], [3, 2, 1])
        self.assertArrays([1, 2, 3], frame._event_times)
        self.assertArrays([3, 2, 1], frame._event_amplitudes)

        frame.clear()
        self.assertArrays([], frame._event_times)
        self.assertArrays([], frame._event_amplitudes)

    def test_get_period(self):
        frame = Frame(1.2, 1)
        self.assertEqual(frame.get_period, 1)

    def test_get_start_time(self):
        frame = Frame(0.1, 3)
        self.assertEqual(frame.get_start_time, 0.1)

    def test_get_events(self):
        frame = Frame(3.1, 2)
        frame.add_events([1, 2, 3], [3, 2, 1])
        self.assertArrays([1, 2, 3], frame.get_event_times)
        self.assertArrays([3, 2, 1], frame.get_event_amplitudes)


if __name__ == '__main__':
    unittest.main()





class Frame(object):
    """
    A frame objects contains event data
    for a period of time for a single detector.
    Methods:
    - add events: adds events to the frame.
    - clear: delects all event information from frame.

    Properties:
    - get_period: gets the period for the frame.
    - get_start_time: gets the start time for the frame.
    - get_event_times: gets the time stamps for the events.
    - get_event_amplitudes: gets the amplitudes for the events.
    """
    def __init__(self, start_time: float, period: int):
        """
        Creates an instance of a frame object.
        :param start_time: is the time (float) since the start of run.
        :param period: the period number for the frame.
        """
        self._start = start_time
        self._period = period

        self._event_times = []
        self._event_amplitudes = []

    def add_events(self, times, amps):
        """
        Adds events to the frame
        :param times: the time stamps for the events.
        :param amps: the amplitudes for the events.
        """
        self._event_times = times
        self._event_amplitudes = amps

    def clear(self):
        """
        Clears all data from the frame.
        """
        self._event_times = []
        self._event_amplitudes = []

    @property
    def get_period(self):
        """
        :returns: the period of the frame.
        """
        return self._period

    @property
    def get_start_time(self):
        """
        :returns: the start time of the frame.
        """
        return self._start

    @property
    def get_event_times(self):
        """
        :returns: the time stamps of the events
        """
        return self._event_times

    @property
    def get_event_amplitudes(self):
        """
        :returns: the event's amplitudes.
        """
        return self._event_amplitudes



from MuonDataLib.data.hdf5 import HDF5
from MuonDataLib.test_helpers.unit_test import TestHelper

import h5py
import unittest
import os


FILENAME = 'hdf5_test.nxs'


class HDF5Test(TestHelper):

    def test_save_str(self):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_str('unit', 'test', group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['unit'])
            self.assertString(group, 'unit', 'test')

        os.remove(FILENAME)

    def bad_str_value(self, value):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_str('unit', value, group)

        os.remove(FILENAME)

    def test_save_str_bad_value_int(self):
        self.bad_str_value(5)

    def test_save_str_bad_value_int_list(self):
        self.bad_str_value([5])

    def test_save_str_bad_value_float(self):
        self.bad_str_value(5.2)

    def test_save_str_bad_value_float_list(self):
        self.bad_str_value([5.2])

    def test_save_float(self):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_float('mean', 2.35, group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['mean'])
            self.assertArrays(group['mean'], [2.35])

        os.remove(FILENAME)

    def float_bad_value(self, value):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_float('mean', value, group)

        os.remove(FILENAME)

    def test_save_float_bad_value_string(self):
        self.float_bad_value('test')

    def test_save_float_bad_value_int(self):
        self.float_bad_value(2)

    def test_save_float_bad_value_int_list(self):
        self.float_bad_value([2])

    def test_save_float_bad_value_float_list(self):
        self.float_bad_value([2.3])

    def test_save_int(self):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_int('version', 5, group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['version'])
            self.assertArrays(group['version'], [5])

        os.remove(FILENAME)

    def int_bad_value(self, value):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_int('version', value, group)

        os.remove(FILENAME)

    def test_int_bad_value_string(self):
        self.int_bad_value('test')

    def test_int_bad_value_float(self):
        self.int_bad_value(1.2)

    def test_int_bad_value_list_int(self):
        self.int_bad_value([3])

    def test_int_bad_value_list_float(self):
        self.int_bad_value([1.2])

    def test_save_int_array(self):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_int_array('count down',
                                [5, 4, 3, 2], group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['count down'])
            self.assertArrays(group['count down'],
                              [5, 4, 3, 2])

        os.remove(FILENAME)

    def int_array_bad_value(self, value):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_int_array('count down', value, group)
        os.remove(FILENAME)

    def test_int_array_bad_value_string(self):
        self.int_array_bad_value('test')

    def test_int_array_bad_value_int(self):
        self.int_array_bad_value(1)

    def test_int_array_bad_value_float(self):
        self.int_array_bad_value(2.3)

    def test_int_array_bad_value_list_float(self):
        self.int_array_bad_value([2.3])

    def test_save_float_array(self):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_float_array('stats', [0.018, 0.039, 0.015], group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['stats'])
            self.assertArrays(group['stats'], [0.018, 0.039, 0.015])

        os.remove(FILENAME)

    def float_array_bad_value(self, value):
        hdf5 = HDF5()

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_float_array('stats', value, group)

        os.remove(FILENAME)

    def test_float_array_bad_value_string(self):
        self.float_array_bad_value('test')

    def test_float_array_bad_value_int(self):
        self.float_array_bad_value(1)

    def test_float_array_bad_value_float(self):
        self.float_array_bad_value(1.2)

    def test_float_array_bad_value_int_array(self):
        self.float_array_bad_value([1])

    def test_save_counts_array(self):
        hdf5 = HDF5()
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist):
                counts[p].append([j for j in range(N_x)])

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            hdf5.save_counts_array('counts', N_p, N_hist, N_x, counts, group)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['data'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['counts'])
            self.assertArrays(group['counts'], counts)

        os.remove(FILENAME)

    def test_save_counts_array_bad_period_length(self):
        hdf5 = HDF5()
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p + 1):
            counts.append([])
            for i in range(N_hist):
                counts[p].append([j for j in range(N_x)])

        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_counts_array('counts',
                                       N_p,
                                       N_hist,
                                       N_x,
                                       counts,
                                       group)

        os.remove(FILENAME)

    def counts_bad_value(self, N_p, N_hist, N_x, counts):
        hdf5 = HDF5()
        with h5py.File(FILENAME, 'w') as file:
            group = file.require_group('data')
            with self.assertRaises(ValueError):
                hdf5.save_counts_array('counts',
                                       N_p,
                                       N_hist,
                                       N_x,
                                       counts,
                                       group)

        os.remove(FILENAME)

    def test_save_counts_array_bad_spec_length(self):
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist - 1):
                counts[p].append([j for j in range(N_x + 1)])
        self.counts_bad_value(N_p, N_hist, N_x, counts)

    def test_save_counts_array_bad_x_length(self):
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist):
                counts[p].append([j for j in range(N_x + 1)])

        self.counts_bad_value(N_p, N_hist, N_x, counts)

    def test_save_counts_array_bad_value_str(self):
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist):
                counts[p].append(['count' for j in range(N_x)])

        self.counts_bad_value(N_p, N_hist, N_x, counts)

    def test_save_counts_array_bad_value_float(self):
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist):
                counts[p].append([1.2 for j in range(N_x)])

        self.counts_bad_value(N_p, N_hist, N_x, counts)

    def test_save_counts_array_bad_value_list(self):
        counts = []
        N_p = 2
        N_hist = 3
        N_x = 4

        for p in range(N_p):
            counts.append([])
            for i in range(N_hist):
                counts[p].append([[j] for j in range(N_x)])

        self.counts_bad_value(N_p, N_hist, N_x, counts)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.utils import (INT32,
                                    stype)
from MuonDataLib.data.hdf5 import HDF5


class Detector_1(HDF5):
    """
    A class for storing the data associated with the detector 1 group
    in a muon NXS v2 file.
    """
    def __init__(self, resolution, raw_time, spec_i,
                 counts, inst, t0, first, last):
        super().__init__()
        """
        Stores the data for the detector 1 group
        :param resolution: the time resolution (micro s)
        :param raw_time: the uncorrected time
        :param spec_i: the spectrum indices
        :param counts: the counts for the data (period, spec, time)
        :param inst: the instrument name
        :param t0: the time zero value
        :param first: the first good bin
        :param last: the last good bin
        """
        self._dict['resolution'] = resolution
        self._dict['raw_time'] = raw_time
        self._dict['spectrum_index'] = spec_i
        self._dict['inst'] = inst
        self._dict['time_zero'] = t0
        self._dict['first_good'] = first
        self._dict['last_good'] = last

        # this will need to change for events ...
        self._dict['counts'] = counts

        self.N_x = len(counts[0][0])
        self.N_hist = len(counts[0])
        self.N_periods = len(counts)

    @property
    def resolution(self):
        """
        Get the resolution in pico seconds
        :return the resolution in pico seconds
        """
        return int(self._dict['resolution']*1e6)

    def save_nxs2(self, file):
        """
        Save the detector 1 values for a muon NXS v2 file
        :param file: the open file to write the data to
        """
        tmp = file.require_group('raw_data_1')
        tmp = tmp.require_group('instrument')
        tmp.attrs['NX_class'] = 'NXinstrument'
        tmp = tmp.require_group('detector_1')
        tmp.attrs['NX_class'] = 'NXdetector'

        resolution = self.save_int('resolution', self.resolution, tmp)
        resolution.attrs.create('units', 'picoseconds'.encode(), dtype='S11')

        raw = self.save_float_array('raw_time', self._dict['raw_time'], tmp)
        raw.attrs.create('units', 'microseconds'.encode(), dtype='S12')
        raw.attrs.create('long_name', 'time'.encode(), dtype='S4')

        self.save_int_array('spectrum_index',
                            self._dict['spectrum_index'],
                            tmp)

        counts = self.save_counts_array('counts', self.N_periods,
                                        self.N_hist, self.N_x,
                                        self._dict['counts'], tmp)
        counts.attrs.create('axes',
                            ('[period index, '
                             'spectrum index, '
                             'raw time bin]').encode(),
                            dtype='S45')
        counts.attrs.create('long_name', self._dict['inst'].encode(),
                            dtype=stype(self._dict['inst']))
        counts.attrs.create('t0_bin', self._dict['time_zero'], dtype=INT32)

        first_bin = int(self._dict['first_good']/self._dict['resolution'])
        counts.attrs.create('first_good_bin',
                            first_bin,
                            dtype=INT32)

        last_bin = int(self._dict['last_good']/self._dict['resolution'])
        counts.attrs.create('last_good_bin',
                            last_bin,
                            dtype=INT32)


def read_detector1_from_histogram(file):
    """
    A method to read the detector 1 values from a
    muon NXS v2 file.
    :param: the open file to read the values from
    :return: a Detector_1 object with the values loaded
    """
    tmp = file['raw_data_1']['instrument']['detector_1']

    # convert to micro seconds
    resolution = tmp['resolution'][0] / 1.e6

    raw_time = tmp['raw_time'][:]
    spec = tmp['spectrum_index'][:]

    tmp = tmp['counts']
    inst = tmp.attrs['long_name'].decode()
    first_good = tmp.attrs['first_good_bin'] * resolution
    last_good = tmp.attrs['last_good_bin'] * resolution
    t0 = tmp.attrs['t0_bin']
    counts = tmp[:]

    # not used...
    # direction =  tmp['orientation'][0].decode()
    # tmp = file['raw_data_1']['detector_1']
    # needed for mantid ?
    # dead_time = tmp['dead_time'][:]
    # grouping = tmp['grouping'][:]
    # time_zero = tmp['time_zero'][:]

    return Detector_1(resolution,
                      raw_time,
                      spec,
                      counts,
                      inst, t0,
                      first_good,
                      last_good)



from MuonDataLib.data.utils import (INT32, FLOAT32,
                                    stype)

import numpy as np


def is_list(values):
    return isinstance(values, np.ndarray) or isinstance(values, list)


def is_int(value):
    return isinstance(value, int) or isinstance(value, INT32)


def is_float(value):
    return isinstance(value, float) or isinstance(value, FLOAT32)


class HDF5(object):
    """
    A wrapper object to make it easier to write
    muon nexus v2 files
    """
    def __init__(self):
        """
        Create an empty dict of values
        """
        self._dict = {}

    def save_str(self, name, string, group):
        """
        Save a string to the relevant part of the nexus file
        :param name: the name used to reference the value in the file
        :param string: the string to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if isinstance(string, str):
            dtype = stype(string)
            return group.require_dataset(name=name,
                                         shape=(1),
                                         data=np.array([string.encode()],
                                                       dtype=dtype),
                                         dtype=dtype)
        else:
            raise ValueError(f'{string} is not a string')

    def save_float(self, name, value, group):
        """
        Save a float value to the relevant part of the nexus file
        :param name: the name used to reference the value in the file
        :param value: the float value to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if is_float(value):
            return group.require_dataset(name=name,
                                         shape=(1),
                                         data=[value],
                                         dtype=FLOAT32)
        else:
            raise ValueError(f'{value} is a {type(value)}, not a float')

    def save_int(self, name, value, group):
        """
        Save an int value to the relevant part of the nexus file
        :param name: the name used to reference the value in the file
        :param value: the int to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if is_int(value):
            return group.require_dataset(name=name,
                                         shape=(1),
                                         data=[value],
                                         dtype=INT32)
        else:
            raise ValueError(f'{value} is a {type(value)}, not an integer')

    def save_int_array(self, name, values, group):
        """
        Save an array of ints to the relevant part of the nexus file
        :param name: the name used to reference the value in the file
        :param values: the int values to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if is_list(values) and is_int(values[0]):
            return group.require_dataset(name=name,
                                         shape=len(values),
                                         data=values,
                                         dtype=INT32)
        else:
            raise ValueError(f'{values} is a {type(values)}, '
                             'not a list of ints')

    def save_float_array(self, name, values, group):
        """
        Save an array of float values to the relevant part of the nexus file
        :param name: the name used to reference the value in the file
        :param values: the float values to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if is_list(values) and is_float(values[0]):
            return group.require_dataset(name=name,
                                         shape=len(values),
                                         data=values,
                                         dtype=FLOAT32)
        else:
            raise ValueError(f'{values} is a {type(values)}, '
                             'not a list of floats')

    def save_counts_array(self, name, N_periods, N_hist, N_x, values, group):
        """
        Save the counts array to the relevant part of the nexus file
        The counts are (period #, spec #, time values)
        It is not practical to test every value, so just do the first.
        :param name: the name used to reference the value in the file
        :param N_periods: the number of periods
        :param N_hist: the number of histograms
        :param N_x: the number of x (time) values
        :param values: the count values to store in the file
        :param group: the location (nexus group) to save the data to
        :return: the nexus dataset object
        """
        if len(values) != N_periods:
            raise ValueError(f'The length of counts is {len(values)} '
                             'and should be of length {N_periods}')

        elif len(values[0]) != N_hist:
            raise ValueError(f'The number of spectra is {len(values[0])} '
                             'and should be {N_hist}')

        elif len(values[0][0]) != N_x:
            raise ValueError(f'The length of x data is {len(values[0][0])} '
                             'and should be of length {N_x}')

        elif is_int(values[0][0][0]):
            return group.require_dataset(name=name,
                                         shape=(N_periods, N_hist, N_x),
                                         data=values,
                                         dtype=INT32)
        else:
            raise ValueError('Should contain integers in count data, '
                             f'not {type(values[0][0][0])}')



from MuonDataLib.data.sample import (Sample,
                                     read_sample_from_histogram)

from MuonDataLib.test_helpers.unit_test import TestHelper
import h5py
import unittest
import os


FILENAME = 'sample_test.nxs'


def create_data():
    return Sample('ISIS2', 23.1, 12.2, 13.3, 5.1, 312.6, 'Si')


class SampleTest(TestHelper):

    def test_sample_object_stores_correct_info(self):
        """
        Check the class stores data correctly
        """
        sample = create_data()

        self.assertEqual(sample._dict['ID'], 'ISIS2')
        self.assertEqual(sample._dict['name'], 'Si')
        self.assertAlmostEqual(sample._dict['thickness'], 23.1, 3)
        self.assertAlmostEqual(sample._dict['height'], 12.2, 3)
        self.assertAlmostEqual(sample._dict['width'], 13.3, 3)
        self.assertAlmostEqual(sample._dict['B_field'], 5.1, 3)
        self.assertAlmostEqual(sample._dict['Temp'], 312.6, 3)

    def test_sample_object_saves_correct_info(self):
        """
        Test that the class can save to a nexus file
        correctly
        """
        sample = create_data()

        with h5py.File(FILENAME, 'w') as file:
            sample.save_nxs2(file)

        with h5py.File(FILENAME, 'r') as file:
            keys = self.compare_keys(file, ['raw_data_1'])
            group = file[keys[0]]

            keys = self.compare_keys(group, ['sample'])
            group = group[keys[0]]
            self.assertEqual(group.attrs['NX_class'], 'NXsample')

            keys = self.compare_keys(group, ['id',
                                             'name',
                                             'thickness',
                                             'height',
                                             'width',
                                             'magnetic_field',
                                             'temperature'])
            self.assertString(group, 'id', 'ISIS2')
            self.assertString(group, 'name', 'Si')
            self.assertArrays(group['thickness'], [23.1])
            self.assertArrays(group['height'], [12.2])
            self.assertArrays(group['width'], [13.3])
            self.assertArrays(group['magnetic_field'], [5.1])
            self.assertArrays(group['temperature'], [312.6])
        os.remove(FILENAME)

    def test_load_sample_gets_correct_info(self):
        """
        Check load method gets the correct information.
        The above tests prove that the information
        stored is correct and that it is correctly
        written to file.
        """
        sample = create_data()

        with h5py.File(FILENAME, 'w') as file:
            sample.save_nxs2(file)

        del sample

        with h5py.File(FILENAME, 'r') as file:
            load_sample = read_sample_from_histogram(file)

        self.assertEqual(load_sample._dict['ID'], 'ISIS2')
        self.assertEqual(load_sample._dict['name'], 'Si')
        self.assertAlmostEqual(load_sample._dict['thickness'], 23.1, 3)
        self.assertAlmostEqual(load_sample._dict['height'], 12.2, 3)
        self.assertAlmostEqual(load_sample._dict['width'], 13.3, 3)
        self.assertAlmostEqual(load_sample._dict['B_field'], 5.1, 3)
        self.assertAlmostEqual(load_sample._dict['Temp'], 312.6, 3)

        os.remove(FILENAME)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.test_helpers.unit_test import TestHelper
import unittest


class UnitTest(unittest.TestCase):

    def test_compare_keys(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        """
        nexus = TestHelper()
        data = {'sample': 1, 'source': 2, 'user': 3}
        ref = ['sample', 'source', 'user']
        # this has an assertion within it
        nexus.compare_keys(data, ref)

    def test_compare_keys_bad_case(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        """
        nexus = TestHelper()
        data = {'sample': 1, 'Source': 2, 'user': 3}
        ref = ['sample', 'source', 'user']
        with self.assertRaises(AssertionError):
            nexus.compare_keys(data, ref)

    def test_compare_keys_repeated_key(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        """
        nexus = TestHelper()
        data = {'sample': 1, 'Source': 2, 'user': 3}
        ref = ['sample', 'source', 'sample']
        with self.assertRaises(AssertionError):
            nexus.compare_keys(data, ref)

    def test_compare_keys_missing_key(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        """
        nexus = TestHelper()
        data = {'sample': 1, 'Source': 2, 'user': 3}
        ref = ['sample', 'user']
        with self.assertRaises(AssertionError):
            nexus.compare_keys(data, ref)

    def test_assertString(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        The dicts have string keys and the
        values are a list of encoded strings.
        """
        nexus = TestHelper()

        data = {'name': ['unit test'.encode()], 'version': ['1.1.2'.encode()]}
        # contains an assert
        nexus.assertString(data, 'name', 'unit test')
        nexus.assertString(data, 'version', '1.1.2')

    def test_assertString_not_encoded(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        The dicts have string keys and the
        values are a list of encoded strings.
        """
        nexus = TestHelper()

        data = {'name': ['unit test'], 'version': ['1.1.2']}

        with self.assertRaises(AttributeError):
            nexus.assertString(data, 'name', 'unit test')
            nexus.assertString(data, 'version', '1.1.2')

    def test_assertString_missing_key(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        The dicts have string keys and the
        values are a list of encoded strings.
        """
        nexus = TestHelper()

        data = {'name': ['unit test'], 'version': ['1.1.2']}

        with self.assertRaises(KeyError):
            nexus.assertString(data, 'python', 'unit test')

    def test_assertString_value_does_not_match(self):
        """
        Since the hdf5 reader returns a dict
        object, we can pass just a dict
        to represent the readers results.
        The dicts have string keys and the
        values are a list of encoded strings.
        """
        nexus = TestHelper()

        data = {'name': ['unit test'.encode()], 'version': ['1.1.2'.encode()]}

        with self.assertRaises(AssertionError):
            nexus.assertString(data, 'name', 'Unit test')
            nexus.assertString(data, 'version', '2.1.2')

    def test_assertArrays_no_recursion(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [1, 2, 3]
        ref = [1, 2, 3]
        nexus = TestHelper()

        nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_diff_len(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [1, 2, 3, 4]
        ref = [1, 2, 3]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_mismatch(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [1, 2, 3.01]
        ref = [1, 2, 3]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)

    def test_assertArrays_one_recursion(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [[1, 2, 3], [4, 5, 6]]
        ref = [[1, 2, 3], [4, 5, 6]]
        nexus = TestHelper()

        nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_diff_len_1(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [[1, 2, 3, 4], [4, 5, 6]]
        ref = [[1, 2, 3], [4, 5, 6]]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_diff_len_2(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [[1, 2, 3], [4, 5, 6, 7]]
        ref = [[1, 2, 3], [4, 5, 6]]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_mismatch_1(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [[1, 2, 3.01], [4, 5, 6]]
        ref = [[1, 2, 3], [4, 5, 6]]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)

    def test_assertArrays_no_recursion_mismatch_2(self):
        """
        The assertArrays method is a recursive so to test it
        we will use something akin to proof by induction.
        We will show it does/doesn't works without any recursion (n=1), then
        that it does/doesn't work with a single recursion (n+1). Therefore,
        it will work for an arbitary number of recursions.
        """
        array = [[1, 2, 3], [4, 5, 5.2]]
        ref = [[1, 2, 3], [4, 5, 6]]
        nexus = TestHelper()
        with self.assertRaises(AssertionError):
            nexus.assertArrays(array, ref)


if __name__ == '__main__':
    unittest.main()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import pathlib
import sys


sys.path.insert(0, pathlib.Path(__file__).parents[2].resolve().as_posix())


project = 'MuonDataLib'
copyright = '2023, Anthony Lim'
author = 'Anthony Lim'
release = '0.0.1'
build_triggered = True

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.autosummary']

templates_path = ['_templates']
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'classic'



import argparse
import sys
from conda_dict_to_yml import write_conda_yml_from_dict


versions = ['3.8', '3.9', '3.10', '3.11']


def get_input():
    """
    get the python version from the command line arg
    :returns the Python version
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('version',
                        help='the Python version'
                        ' (3.8, 3.9, 3.10, 3.11)', type=str)
    args = parser.parse_args()

    if args.version not in versions:
        raise ValueError(str(args.version) + ' is not a supported'
                                             ' Python version.')
    print(args.version)
    return args.version


def get_info(version):
    """
    Gets the yml_dict and file name from the OS
    :param version: the Python version
    :Return a dict of contents for the yml file and file name
    """
    yml_dict = create_default(version)
    file_name = str(yml_dict["name"])+'.yml'
    return yml_dict, file_name


"""
Here we list the conda recipes.
We start with a default.
Can add changes for specific OS's
"""


def create_default(version):
    """
    The default yml file for all OS's.
    Should only change these values if there is a
    good reason.
    :param version: Python version to build (exclude patch)
    """
    default_yml = {}

    pip_dict = {}

    default_yml['name'] = 'MuonDataLib-dev'
    default_yml['channels'] = 'conda-forge'
    default_yml['dependencies'] = {'python': '=' + version + '.*',
                                   'numpy': '',
                                   'plotly': '',
                                   'pytest': '',
                                   'pre-commit': '>=2.15',
                                   'pip': pip_dict}
    return default_yml


if __name__ == "__main__":
    try:
        version = get_input()
        yml_dict, file_name = get_info(version)
        with open(file_name, 'w') as outfile:
            write_conda_yml_from_dict(yml_dict, outfile)

    except ValueError:
        error = sys.exc_info()[1]
        print(error)



from MuonDataLib.data.hdf5 import HDF5


class Periods(HDF5):
    """
    A class to store the period informtaion for muon data
    """
    def __init__(self, number, labels, p_type, requested,
                 raw, output, counts, sequences):
        """
        A class to store the period data needed for a muon nexus v2 file
        :param number: the number of periods
        :param labels: a string of the period labels
        :param p_type: an int array representing the type of period
        :param requested: the number of requested frames
        :param raw: the number of raw frames
        :param output: an int array of the outputs
        :param counts: a float array of the total counts
        :param sequences: an int array of the sequences
        """
        super().__init__()
        self._dict['number'] = number
        self._dict['labels'] = labels
        self._dict['type'] = p_type
        self._dict['requested'] = requested
        self._dict['raw'] = raw
        self._dict['output'] = output
        self._dict['counts'] = counts
        self._dict['sequences'] = sequences

    def save_nxs2(self, file):
        """
        A method to save the periods information as a muon
        nexus v2 file
        :param file: the open file to write to
        """
        tmp = file.require_group('raw_data_1')
        tmp = tmp.require_group('periods')

        tmp.attrs['NX_class'] = 'NXperiod'
        self.save_int('number', self._dict['number'], tmp)
        self.save_int_array('sequences', self._dict['sequences'], tmp)
        self.save_str('labels', self._dict['labels'], tmp)
        self.save_int_array('type', self._dict['type'], tmp)
        self.save_int_array('frames_requested', self._dict['requested'], tmp)
        self.save_int_array('raw_frames', self._dict['raw'], tmp)
        self.save_int_array('output', self._dict['output'], tmp)
        self.save_float_array('total_counts', self._dict['counts'], tmp)


def read_periods_from_histogram(file):
    """
    A method for reading the period information
    a nexus v2 histogram file
    :param file: the open file to read from
    :return: the Periods object
    """
    tmp = file['raw_data_1']['periods']

    return Periods(number=tmp['number'][:][0],
                   labels=tmp['labels'][:][0].decode(),
                   p_type=tmp['type'][:],
                   requested=tmp['frames_requested'][:],
                   raw=tmp['raw_frames'][:],
                   output=tmp['output'][:],
                   counts=tmp['total_counts'][:],
                   sequences=tmp['sequences'][:])



from MuonDataLib.data.loader.load_event import LoadEventData
import unittest
import time
import os
import numpy as np


DATADIR = os.path.join(os.path.dirname(__file__),
                       '..',
                       'data_files')
INFILE = os.path.join(DATADIR, 'complete_run.nxs')
REFFILE = None
OUTFILE = 'single_period_event_test.nxs'


class SinglePeriodEventTest(unittest.TestCase):

    def test_run_time(self):
        """
        This is fairly fast code, so will
        do 100 workflows to get some statistics.
        """
        times = []
        for j in range(100):
            start = time.time()

            data = LoadEventData()
            data.load_data(INFILE)
            hist, bins = data.get_histograms()
            times.append(time.time() - start)
            del data
        # the requirment is that its less than 5 seconds
        self.assertLess(np.mean(times), 0.1)
        self.assertLess(np.max(times), 1.0)
        self.assertLess(np.max(times), 5.00)

    def test_reload_run_time(self):
        times = []
        full = os.path.join(DATADIR, 'complete_run.nxs')
        part = os.path.join(DATADIR, 'partial_run.nxs')
        for j in range(100):

            data = LoadEventData()
            data.load_data(part)
            data._file_name = full
            # only want to time the reload
            start = time.time()
            data.reload_data()
            hist, bins = data.get_histograms()
            times.append(time.time() - start)
            del data
        # the requirment is that its less than 5 seconds
        self.assertLess(np.mean(times), 0.1)
        self.assertLess(np.max(times), 1.0)
        self.assertLess(np.max(times), 5.00)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.loader.load_nxs2 import load_nxs2
import unittest
import time
import os
import numpy as np
import filecmp


DATADIR = os.path.join(os.path.dirname(__file__),
                       '..',
                       'data_files')
INFILE = os.path.join(DATADIR, 'HIFI00180594.nxs')
REFFILE = os.path.join(DATADIR, 'HIFI42.nxs')
OUTFILE = 'single_period_test.nxs'


class SinglePeriodSystemTest(unittest.TestCase):

    def test_run_time(self):
        """
        This is fairly fast code, so will
        do 100 workflows to get some statistics.
        """
        times = []
        for j in range(100):
            start = time.time()
            data = load_nxs2(INFILE)
            data.save_histograms(OUTFILE)
            times.append(time.time() - start)
            os.remove(OUTFILE)
            del data
        # the requirment is that its less than 5 seconds
        self.assertLess(np.mean(times), 0.1)
        self.assertLess(np.max(times), 1.0)
        self.assertLess(np.max(times), 5.00)

    def test_saved_data(self):
        data = load_nxs2(INFILE)
        data.save_histograms(OUTFILE)

        self.assertTrue(filecmp.cmp(REFFILE, OUTFILE, shallow=False),
                        msg=('The single period nexus file does not '
                             'match the reference file'))

        os.remove(OUTFILE)


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.hdf5 import HDF5


class Sample(HDF5):
    """
    A class for the sample information needed for a muon nexus v2 file
    """
    def __init__(self, ID, thickness, height, width, B_field, Temp, name):
        """
        The sample information for a muon nexus v2 file
        :param ID: the sample ID
        :param thickness: the thickness of the sample
        :param height: the height of the sample
        :param width: the width of the sample
        :param B_field: the magnetic field being applied to the sample
        :param Temp: the applied temperature to the sample
        :param name: the name of the sample
        """
        super().__init__()
        self._dict['ID'] = ID
        self._dict['thickness'] = thickness
        self._dict['height'] = height
        self._dict['width'] = width
        self._dict['B_field'] = B_field
        self._dict['Temp'] = Temp
        self._dict['name'] = name

    def save_nxs2(self, file):
        """
        A method to write the sample information into
        a muon nexus v2 file
        :param file: the open file to write the data to
        """
        tmp = file.require_group('raw_data_1')
        tmp = tmp.require_group('sample')
        tmp.attrs['NX_class'] = 'NXsample'

        self.save_str('id', self._dict['ID'], tmp)
        self.save_str('name', self._dict['name'], tmp)
        self.save_float('thickness', self._dict['thickness'], tmp)
        self.save_float('height', self._dict['height'], tmp)
        self.save_float('width', self._dict['width'], tmp)
        self.save_float('magnetic_field', self._dict['B_field'], tmp)
        self.save_float('temperature', self._dict['Temp'], tmp)


def read_sample_from_histogram(file):
    """
    A function to read the sample information from
    a muon nexus v2 file
    :param file: the open file to read from
    :return: the Sample information
    """
    tmp = file['raw_data_1']['sample']
    return Sample(tmp['id'][0].decode(),
                  tmp['thickness'][0],
                  tmp['height'][0],
                  tmp['width'][0],
                  tmp['magnetic_field'][0],
                  tmp['temperature'][0],
                  tmp['name'][0].decode())



from MuonDataLib.data.loader.load_nxs2 import load_nxs2

import unittest
from unittest import mock
import os


class LoadNxs2Test(unittest.TestCase):

    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_sample_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_raw_data_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_source_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_user_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_periods_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.read_detector1_from_histogram')
    @mock.patch('MuonDataLib.data.loader.'
                'load_nxs2.MuonData')
    def test_load_nxs2(self,
                       muon_data,
                       detector_1,
                       periods,
                       user,
                       source,
                       raw_data,
                       sample):
        """
        Going to check that the function
        is called correctly by using mocks.
        The creation of the muon data
        object is covered by its own tests.
        """
        file = os.path.join(os.path.dirname(__file__),
                            '..',
                            '..',
                            'data_files',
                            'HIFI00180594.nxs')
        load_nxs2(file)

        # check the read functions are called
        sample.assert_called_once()
        raw_data.assert_called_once()
        source.assert_called_once()
        user.assert_called_once()
        periods.assert_called_once()
        detector_1.assert_called_once()

        # check muon data gets the correct read functions
        muon_data.assert_called_once_with(sample=sample(),
                                          raw_data=raw_data(),
                                          source=source(),
                                          user=user(),
                                          periods=periods(),
                                          detector1=detector_1())


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.utils import (convert_date_for_NXS,
                                    convert_date,
                                    stype)
import unittest
import datetime


class UtilsTest(unittest.TestCase):

    def test_stype(self):
        self.assertEqual(stype('testing'), 'S8')

    def test_convert_date(self):
        date = '2018-12-24T13:32:01'
        date = convert_date(date)
        self.assertEqual(date, datetime.datetime(2018, 12, 24, 13, 32, 1))

    def test_convert_date_for_NXS(self):
        date = datetime.datetime(2018, 12, 24, 13, 32, 1)
        date = convert_date_for_NXS(date)
        self.assertEqual(date, '2018-12-24T13:32:01')


if __name__ == '__main__':
    unittest.main()



from MuonDataLib.data.hdf5 import HDF5


class User(HDF5):
    """
    A simple class to store the user information
    needed for a muon nexus v2 file
    """
    def __init__(self, name, affiliation):
        """
        Store the user information for muon nexus v2 file
        :param name: the user's name
        Param affiliation: the user's affiliation/institution
        """
        super().__init__()
        self._dict['name'] = name
        self._dict['affiliation'] = affiliation

    def save_nxs2(self, file):
        """
        Write the user information to a
        muon nexus v2 file.
        :param file: the open file to write to
        """
        tmp = file.require_group('raw_data_1')
        tmp = tmp.require_group('user_1')
        tmp.attrs['NX_class'] = 'NXuser'

        for key in self._dict.keys():
            self.save_str(key, self._dict[key], tmp)


def read_user_from_histogram(file):
    """
    A function to read the histogram information
    from a muon nexus v2 file
    :param file: the open file to read from
    :return: the user information
    """
    tmp = file['raw_data_1']
    tmp = tmp['user_1']

    return User(tmp['name'][0].decode(),
                tmp['affiliation'][0].decode())



from MuonDataLib.data.events.detector import Detector
import numpy as np


def filter_data(data_list, condition):
    """
    A simple function for filtering a series
    of lists based on the same condition.
    :param data_list: the list of lists to filter.
    :param condition: the filter condition.
    :returns: a list of filtered data.
    """
    indicies = np.where(condition)[0]

    return [np.asarray(data)[indicies] for data in data_list]


class Instrument(object):
    """
    An Instrument is made up of multiple detectors.
    Each Instrument represents a single run.

    Methods:
    - set_bins: set the bins for the histograms.
    - get_histograms: gets the histogram values and bin edges.
    - add_new_frame: adds a new frame to all of the detectors.
    - add_event_data: sorts and adds event data to each detector.
    """
    def __init__(self, start_time, N_det=64):
        """
        Creates an instrument object.
        By default the bins are 0.5 micro seconds.
        :param start_time: this is the date value for the start of the run.
        :param N_det: the number of detectors (default 64)
        """
        self.N_det = N_det
        self._detectors = [Detector(j) for j in range(self.N_det)]
        self._start = start_time
        # need to start counting from 0 after increment
        self._current_frame = -1
        self._current_index = None
        self._bounds = [0, 30]
        self.set_bins(.5)
        # ns to micro sec
        self._unit_conversion = 1.e-3

    def set_bins(self, bin_width):
        """
        Sets bin edges to a constant width from 0 to 30 micro sec.
        :param bin_width: the bin width to use when calculating the
        bin edges.
        """
        self._bin_edges = np.arange(self._bounds[0],
                                    self._bounds[1] + bin_width,
                                    bin_width)

    def get_histogram(self, ID):
        """
        Gets the histogram data for a single detector.
        :param ID: the detector ID to get the histogram for.
        :returns: the heights and bin edges for the histogram.
        """
        return self._detectors[ID].get_histogram(self._bin_edges,
                                                 self._unit_conversion)

    def add_new_frame(self, start_time, period, index):
        """
        Adds a new frame to all of the detectors.
        :param start_time: the start time for the new frame.
        :param period: the period of the new frame.
        :param index: the event index that the new frame starts at.
        """
        self._current_frame += 1
        self._current_index = index
        for j, _ in enumerate(self._detectors):
            self._detectors[j].add_new_frame(start_time, period)

    def add_event_data(self,
                       IDs,
                       all_times,
                       all_amps,
                       periods,
                       start_times,
                       start_indicies):
        """
        Adds event data to the correct frame and detector.
        :param IDs: the detector IDs
        :param all_times: all of the event time stamps.
        :param all_amps: all of the event amplitudes.
        :param periods: all of the periods.
        :param start_times: the start times for the frames.
        :param start_indicies: the index of first event for each frame.
        """
        # list of start indicies for active and new frames
        condition = np.asarray(start_indicies) >= self._current_index
        frame_list, starts, p = filter_data([start_indicies,
                                             start_times,
                                             periods],
                                            condition)

        # loop over frames
        for k in range(len(frame_list)-1):
            current = frame_list[k]
            next_frame = frame_list[k + 1]
            self._add_data(self._current_frame,
                           IDs[current: next_frame],
                           all_times[current: next_frame],
                           all_amps[current: next_frame])
            # will always be a next frame in this loop
            self.add_new_frame(starts[k+1],
                               p[k+1],
                               frame_list[k+1])
        # load data for current frame
        self._add_data(self._current_frame, IDs[self._current_index:],
                       all_times[frame_list[-1]:],
                       all_amps[frame_list[-1]:])

    def _add_data(self, frame, IDs, times, amps):
        """
        Adds event data to the detectors.
        This will filter events to the correct detector.
        :param frame: the frame number to add events to.
        :param IDs: the detector ID list for the events.
        :param times: the event time stamps.
        :param amps: the event amplitudes.
        """
        for j, _ in enumerate(self._detectors):
            # get indicies for the current detector
            indicies = np.where(np.asarray(IDs) == j)[0]
            self._detectors[j].add_events_to_frame(frame,
                                                   np.asarray(times)[indicies],
                                                   np.asarray(amps)[indicies])



from MuonDataLib.data.muon_data import MuonData
import unittest
import os
from unittest import mock


class fake_nxs_part(object):
    def __init__(self):
        self.save_nxs2 = mock.Mock(return_value='moo')

    def assert_called_once(self):
        self.save_nxs2.assert_called_once()


class MuonDataTest(unittest.TestCase):

    def test_MuonData_init(self):
        '''
        Instead of creating the full objects we
        will just use strings. Since we only need
        to check that the dict matches the
        argument correctly.
        For simplicity the strings used match
        the keys in the dict
        '''
        data = MuonData('sample',
                        'raw_data',
                        'source',
                        'user',
                        'periods',
                        'detector_1')

        for key in data._dict.keys():
            self.assertEqual(key, data._dict[key])

    def test_save_histogram(self):
        """
        Want to test that all of the individual
        parts are called when saving. So will use
        mocks.
        """
        sample = fake_nxs_part()
        raw_data = fake_nxs_part()
        source = fake_nxs_part()
        user = fake_nxs_part()
        periods = fake_nxs_part()
        detector_1 = fake_nxs_part()

        data = MuonData(sample,
                        raw_data,
                        source,
                        user,
                        periods,
                        detector_1)
        data.save_histograms('tmp.nxs')

        sample.assert_called_once()
        raw_data.assert_called_once()
        source.assert_called_once()
        user.assert_called_once()
        periods.assert_called_once()
        detector_1.assert_called_once()

        os.remove('tmp.nxs')



from MuonDataLib.data.events.frame import Frame
import numpy as np


class Detector(object):
    """
    The Detector class represents a single
    detector for an instrument.
    A detector records events in frames.

    Methods:
    - add_new_frame
    - add_events_to_frame
    - get_histogram

    """
    def __init__(self, ID):
        """
        Creates a Detector
        :param ID: The identifier for
        the detector (e.g. detector number)
        """
        self._ID = ID
        self._frames = {}
        self._current = -1

    def add_new_frame(self, time, period):
        """
        Adds a new frame to the detector.
        :param time: the start time of
        the frame in ns.
        :param period: the period of the frame.
        """
        time_in_nsec = time
        self._current += 1
        self._frames[self._current] = Frame(time_in_nsec, period)

    def add_events_to_frame(self, frame, times, amps):
        """
        Adds events to a specific frame.
        :param frame: the frame number to add events to.
        :param time: the event time stamps to add
        :param amps: the event amplitudes to add
        """
        self._frames[frame].add_events(times, amps)

    def get_histogram(self, bin_edges, unit_conversion=1):
        """
        Generates a histogram for the detector.
        :param bin_edges: the bin edges to use in the histogram.
        :param unit_conversion: A conversion factor for units (e.g.
        ns to micro seconds).
        :returns: The histogram y values, bin edges.
        """
        events = np.asarray([])
        for frame in self._frames.keys():
            events = np.append(events, self._frames[frame].get_event_times)
        return np.histogram(events * unit_conversion,
                            bins=bin_edges)



import h5py


class MuonData(object):
    """
    A class to store all of the information needed for muon data
    """
    def __init__(self, sample, raw_data, source, user, periods, detector1):
        """
        Creates a store for relevant muon data (defined by nxs v2)
        :param sample: the Sample data needed for nexus v2 file
        :param raw_data: the RawData data needed for nexus v2 file
        :param source: the Source data needed for nexus v2 file
        :param user: the User data needed for nexus v2 file
        :param periods: the Periods data needed for nexus v2 file
        :param detector1: the Detector1 data needed for nexus v2 file
        """
        self._dict = {}
        self._dict['raw_data'] = raw_data
        self._dict['sample'] = sample
        self._dict['source'] = source
        self._dict['user'] = user
        self._dict['periods'] = periods
        self._dict['detector_1'] = detector1

    def save_histograms(self, file_name):
        """
        Method for saving the object to a muon
        nexus v2 histogram file
        :param file_name: the name of the file to save to
        """
        file = h5py.File(file_name, 'w')
        for key in self._dict.keys():
            self._dict[key].save_nxs2(file)
        file.close()
        return



from MuonDataLib.data.events.instrument import Instrument, filter_data
from MuonDataLib.test_helpers.unit_test import TestHelper
from MuonDataLib.test_helpers.utils import extract_event_data
import unittest
import numpy as np


DATE = '2024-05-23T09:03:23.85'


class InstrumentTest(TestHelper):

    def test_filter_data(self):
        data_0 = np.asarray([0, 1, 2, 3, 4, 5])
        data_1 = np.asarray([5, 4, 3, 2, 1, 0])

        filter_0, filter_1 = filter_data([data_0, data_1], data_0 > 2)
        self.assertArrays(filter_0, [3, 4, 5])
        self.assertArrays(filter_1, [2, 1, 0])

    def test_init(self):
        inst = Instrument(DATE, 2)
        self.assertEqual(inst._start, DATE)
        self.assertEqual(inst._current_frame, -1)
        self.assertEqual(inst._current_index, None)
        self.assertEqual(len(inst._detectors), 2)
        self.assertArrays(inst._bin_edges, np.arange(0, 30.5, .5))
        self.assertArrays(inst._bounds, [0, 30])
        self.assertEqual(inst._unit_conversion, 1e-3)

    def test_add_new_frame(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(0.1, 0, 0)
        self.assertEqual(inst._start, DATE)
        self.assertEqual(inst._current_frame, 0)
        self.assertEqual(inst._current_index, 0)

    def assertEvents(self, inst, ID, frame, times, amps, period, start):
        frame = extract_event_data(inst, ID, frame)
        self.assertArrays(frame.get_event_times, times)
        self.assertArrays(frame.get_event_amplitudes, amps)
        self.assertEqual(frame.get_period, period)
        self.assertEqual(frame.get_start_time, start)

    def test_add_data(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(0.1, 0, 0)
        inst._add_data(0,
                       [0, 1, 0, 1],
                       [1, 2, 3, 4],
                       [4, 3, 2, 1])
        self.assertEvents(inst, 0, 0,
                          [1, 3],
                          [4, 2],
                          0, 0.1)

    def test_add_event_data(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(1.2, 0, 0)

        inst.add_event_data([0, 1, 0, 1, 0, 1, 0],
                            [1, 2, 3, 4, 5, 6, 7],
                            [1, 2, 1, 2, 1, 2, 1],
                            [0, 0, 0, 0, 0, 0, 0],
                            [1.2], [0])

        self.assertEvents(inst, 0, 0,
                          [1, 3, 5, 7],
                          [1, 1, 1, 1],
                          0, 1.2)

        self.assertEvents(inst, 1, 0,
                          [2, 4, 6],
                          [2, 2, 2],
                          0, 1.2)

    def test_extend_current_frame(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(1.2, 0, 0)

        # just provide some data
        inst.add_event_data([0, 1, 0],
                            [1, 2, 3],
                            [1, 2, 1],
                            [0, 0, 0],
                            [1.2], [0])

        self.assertEvents(inst, 0, 0,
                          [1, 3],
                          [1, 1],
                          0, 1.2)

        self.assertEvents(inst, 1, 0,
                          [2],
                          [2],
                          0, 1.2)

        # update with more data
        inst.add_event_data([0, 1, 0, 1, 0, 1, 0],
                            [1, 2, 3, 4, 5, 6, 7],
                            [1, 2, 1, 2, 1, 2, 1],
                            [0, 0, 0, 0, 0, 0, 0],
                            [1.2], [0])

        self.assertEvents(inst, 0, 0,
                          [1, 3, 5, 7],
                          [1, 1, 1, 1],
                          0, 1.2)

        self.assertEvents(inst, 1, 0,
                          [2, 4, 6],
                          [2, 2, 2],
                          0, 1.2)

    def test_add_2_frames(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(1.2, 0, 0)

        inst.add_event_data([0, 1, 0, 1, 0, 1, 0, 1],
                            [1, 2, 3, 4, 5, 6, 7, 8],
                            [1, 2, 1, 2, 2, 1, 2, 1],
                            [0, 0],
                            [1.2, 4.1],
                            [0, 4])

        self.assertEvents(inst, 0, 0,
                          [1, 3],
                          [1, 1],
                          0, 1.2)

        self.assertEvents(inst, 1, 0,
                          [2, 4],
                          [2, 2],
                          0, 1.2)

        # check frame 2
        self.assertEvents(inst, 0, 1,
                          [5, 7],
                          [2, 2],
                          0, 4.1)

        self.assertEvents(inst, 1, 1,
                          [6, 8],
                          [1, 1],
                          0, 4.1)

    def test_add_3_frames(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(1.2, 0, 0)

        inst.add_event_data([0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
                            [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                            [1, 2, 1, 2, 2, 1, 2, 1, 3, 4],
                            [0, 0, 0],
                            [1.2, 4.1, 8.6],
                            [0, 4, 8])

        self.assertEvents(inst, 0, 0,
                          [1, 3],
                          [1, 1],
                          0, 1.2)

        self.assertEvents(inst, 1, 0,
                          [2, 4],
                          [2, 2],
                          0, 1.2)

        # check frame 2
        self.assertEvents(inst, 0, 1,
                          [5, 7],
                          [2, 2],
                          0, 4.1)

        self.assertEvents(inst, 1, 1,
                          [6, 8],
                          [1, 1],
                          0, 4.1)
        # check frame 3
        self.assertEvents(inst, 0, 2,
                          [9],
                          [3],
                          0, 8.6)

        self.assertEvents(inst, 1, 2,
                          [10],
                          [4],
                          0, 8.6)

    def test_set_bins(self):
        inst = Instrument(DATE, 2)
        # change the bounds to make test simpiler
        inst._bounds = [1, 4]
        inst.set_bins(0.5)
        self.assertArrays(inst._bin_edges,
                          [1., 1.5, 2, 2.5, 3, 3.5, 4])

    def test_get_histogram(self):
        inst = Instrument(DATE, 2)
        inst.add_new_frame(1.2, 0, 0)

        inst.add_event_data([0, 1, 0, 1, 0, 1, 0, 1],
                            np.asarray([1, 2, 3, 4, 5, 6, 7, 8])*1.e3,
                            [1, 2, 1, 2, 2, 1, 2, 1],
                            [0, 0],
                            [1.2, 4.1],
                            [0, 4])
        inst._bounds = [0, 10]
        inst.set_bins(1.)
        # check detector 0
        y, x = inst.get_histogram(0)
        self.assertArrays(x, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
        self.assertArrays(y, [0, 1, 0, 1, 0, 1, 0, 1, 0, 0])

        # check detector 1
        y, x = inst.get_histogram(1)
        self.assertArrays(x, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
        self.assertArrays(y, [0, 0, 1, 0, 1, 0, 1, 0, 1, 0])


if __name__ == '__main__':
    unittest.main()
